#include <Wire.h>
#include <IRremote.h>
#include <DS1307RTC.h>

#include <avr/eeprom.h>
#include <FastLED.h>

#define DATA_PIN_0 12   // пин, куда подключена линия данных светодиодов
#define NUM_LEDS 6      // количество светодиодов
CRGB leds_0[NUM_LEDS];

//********************************************************************************************************************
// настройки пламени
#define HUE_GAP 21      // заброс по hue
#define FIRE_STEP 15    // шаг огня
#define HUE_START 80    // начальный цвет огня (0 красный, 80 зелёный, 140 молния, 190 розовый)
#define MIN_BRIGHT 70   // мин. яркость огня
#define MAX_BRIGHT 255  // макс. яркость огня
#define MIN_SAT 245     // мин. насыщенность
#define MAX_SAT 255     // макс. насыщенность

int counter = 0;
bool muted_radio = 0;
//********************************************************************************************************************

#include "defines_radio.h" // В этом файле содержатся все дефайны и функции по работе с радио. Вытащил в отдельный файл для удобства.
#include "pins_setup.h" // В этом файле настройки пинов ввода-вывода

IRrecv irrecv(9); // указываем вывод, к которому подключен приемник
decode_results results;

unsigned char dig[10] = {0b0000, 0b1000, 0b0010, 0b1010, 0b0001, 0b1001, 0b0011, 0b1011, 0b0100, 0b1100};
unsigned char lamps [12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

unsigned char h = 0;
unsigned char m = 0;
unsigned char s = 0;

unsigned char _day = 0;
unsigned char mon = 0;
unsigned char yer = 0;

tmElements_t datetime;

uint8_t EEMEM sec_scroll_addr;
uint8_t EEMEM all_30_scroll_addr;
uint8_t EEMEM tik_tak_addr;
uint8_t EEMEM reverse_addr;
uint8_t EEMEM alarm_is_active_addr;
uint8_t EEMEM defect_addr;

uint8_t old_n_effects = 0;

byte defect;
byte sec_scroll;
byte all_30_scroll;
byte tik_tak;
byte reverse;
byte alarm_is_active;

uint8_t EEMEM alarm_clock_hour_addr;
uint8_t EEMEM alarm_clock_minute_addr;

byte alarm_clock_hour;
byte alarm_clock_minute;

bool radio_on = 0;
bool alarm_is_already_on = 0;
bool chg = 0;

uint16_t EEMEM f1_addr;
uint16_t EEMEM f2_addr;
uint16_t EEMEM f3_addr;
uint16_t EEMEM f4_addr;
uint16_t EEMEM f5_addr;
uint16_t EEMEM f6_addr;

uint16_t f1;
uint16_t f2;
uint16_t f3;
uint16_t f4;
uint16_t f5;
uint16_t f6;

void setup() {
  randomSeed(analogRead(A0));

  FastLED.addLeds<WS2812B, DATA_PIN_0>(leds_0, NUM_LEDS);
  off_backlight ();

  irrecv.enableIRIn(); // запускаем прием

  setup_radio_first ();

  setup_pins ();

  sec_scroll = eeprom_read_byte(&sec_scroll_addr);
  all_30_scroll = eeprom_read_byte(&all_30_scroll_addr);
  tik_tak = eeprom_read_byte(&tik_tak_addr);
  reverse = eeprom_read_byte(&reverse_addr);
  alarm_is_active = eeprom_read_byte(&alarm_is_active_addr);
  defect = eeprom_read_byte(&defect_addr);

  if ((defect != 1) && (defect != 0)) defect = 1;
  if ((sec_scroll != 1) && (sec_scroll != 0)) sec_scroll = 1;
  if ((all_30_scroll != 1) && (all_30_scroll != 0)) all_30_scroll = 1;
  if ((tik_tak != 1) && (tik_tak != 0)) tik_tak = 1;
  if ((reverse != 1) && (reverse != 0)) reverse = 0;
  if ((alarm_is_active != 1) && (alarm_is_active != 0)) alarm_is_active = 0;

  alarm_clock_hour = eeprom_read_byte (&alarm_clock_hour_addr);
  alarm_clock_minute = eeprom_read_byte (&alarm_clock_minute_addr);

  if ((alarm_clock_hour < 0) || (alarm_clock_hour > 23)) alarm_clock_hour = 9;
  if ((alarm_clock_minute < 0) || (alarm_clock_minute > 59)) alarm_clock_minute = 30;

  f1 = eeprom_read_word (&f1_addr);
  f2 = eeprom_read_word (&f2_addr);
  f3 = eeprom_read_word (&f3_addr);
  f4 = eeprom_read_word (&f4_addr);
  f5 = eeprom_read_word (&f5_addr);
  f6 = eeprom_read_word (&f6_addr);
  // инициализация Timer1
  cli(); // отключить глобальные прерывания

  TCCR1A = 0;
  TCCR1B = 0;
  TIMSK1 = 0;

  OCR1A = 65500; // установка регистра совпадения

  TCCR1A = (0 << COM1A1) | (0 << COM1A0) | (0 << COM1B1) | (0 << COM1B0) | (1 << WGM11) | (1 << WGM10);
  TCCR1B = (1 << WGM12) | (1 << WGM13) | (0 << CS12) | (0 << CS11) | (1 << CS10); // включение в CTC режим

  TIMSK1 = (0 << ICIE1) | (0 << OCIE1B) | (1 << OCIE1A) | (1 << TOIE1);

  sei();  // включить глобальные прерывания
}

#include "ovf_cmp.h" // Сюда вытащил прерывание по совпадению и прерывание по переполнению с целью уменьшить основной код



int idex = 0;                //-LED INDEX (0 to LED_COUNT-1
int ihue = 0;                //-HUE (0-255)

int thisdelay = 20;          //-FX LOOPS DELAY VAR
int thisstep = 10;           //-FX LOOPS DELAY VAR
int thissat = 255;           //-FX LOOPS DELAY VAR
int thishue = 255;
int ibright = 0;             //-BRIGHTNESS (0-255)

int bouncedirection = 0;     //-SWITCH FOR COLOR BOUNCE (0-1)

int TOP_INDEX = int(NUM_LEDS / 2);


uint32_t myTimer1 = 0;
bool first_run = 1;
bool setup_mode = 0;
uint32_t bL1 = 0;
uint32_t bL2 = 0;

bool night_cycle = 0;
byte defect_lamp;

bool key_pressed (unsigned char key, unsigned char index, unsigned char tm, bool on_lamp = false) {
  long drebezg = 0;
  while (!digitalRead (key)) { // читаем состояние средней кнопки
    drebezg++;

    if (on_lamp) {
      lamps [index] =   tm / 10;
      lamps [index + 1] = tm % 10;
    }

  }
  if (drebezg > 1000) {
    return 1;
  } else {
    return 0;
  }
}

int i = 0;

long key = 0;

uint8_t base_change;
uint8_t color_rotation_time;
uint8_t color_rotation;
uint8_t hue;

unsigned char color = 0;

static const int wave_time_div = 16;
static const int color_rotation_speed = 8;

unsigned char num_of_effect = 0;

bool temporary_show = 0;
long tmp = 0;
unsigned char scroll_mode = 0;

bool show_date = 0;

byte old_effects = 0;

uint8_t tempera;

void loop() {
  if (first_run) {
    update_time_date ();

    mute_radio ();

    off_backlight ();

    first_run = 0;
  }
  key = return_code ();
  //************************************ ЦИКЛ НАСТРОЙКИ ВРЕМЕНИ ПО НАЖАТИИ КНОПКИ ОК ************************************
  if (key_pressed (A1, 0, 0, false) || (key == 0xFF02FD /*OK_KEY*/)) {

    if (show_date == 1) { // Если переходим в настройку в режиме показа даты, то надо настраивать дату
      bL_to_zero ();
      while (1) {
        if (millis() - bL1 >= 100) {  // ищем разницу (500 мс)
          bL1 = millis();
          lamps [1] = _day % 10;
          lamps [0] = _day / 10;
        }
        if (millis() - bL2 >= 200) {
          bL2 = millis();
          lamps [1] = 10;
          lamps [0] = 10;
        }

        key = return_code ();

        if (key == 0xFF629D /*UP_KEY*/) {
          // beep ();
          _day++;
          if (_day > 31) {
            _day = 1;
          }
          bL_to_zero ();
        }

        if (key == 0xFFA857 /*DOWN_KEY*/) {
          //beep ();
          _day--;
          if (_day < 1) {
            _day = 31;
          }
          bL_to_zero ();
        }

        if (key == 0xFF02FD /*OK_KEY*/) {
          lamps [1] = _day % 10;
          lamps [0] = _day / 10;
          break;
        }
      }
      bL_to_zero ();
      while (1) {
        if (millis() - bL1 >= 100) {  // ищем разницу (500 мс)
          bL1 = millis();
          lamps [3] = mon % 10;
          lamps [2] = mon / 10;
        }
        if (millis() - bL2 >= 200) {
          bL2 = millis();
          lamps [3] = 10;
          lamps [2] = 10;
        }
        key = return_code ();

        if (key == 0xFF629D /*UP_KEY*/) {
          // beep ();
          mon++;
          if (mon > 12) {
            mon = 1;
          }
          bL_to_zero ();
        }

        if (key == 0xFFA857 /*DOWN_KEY*/) {
          // beep ();
          mon--;
          if (mon < 1) {
            mon = 12;
          }
          bL_to_zero ();
        }

        if (key == 0xFF02FD /*OK_KEY*/) {
          lamps [3] = mon % 10;
          lamps [2] = mon / 10;
          break;
        }
      }
      bL_to_zero ();
      while (1) {
        if (millis() - bL1 >= 100) {  // ищем разницу (500 мс)
          bL1 = millis();
          lamps [5] = yer % 10;
          lamps [4] = yer / 10;
        }
        if (millis() - bL2 >= 200) {
          bL2 = millis();
          lamps [5] = 10;
          lamps [4] = 10;
        }

        key = return_code ();

        if (key == 0xFF629D /*UP_KEY*/) {
          //beep ();
          yer++;
          if (yer > 99) {
            yer = 20;
          }
          bL_to_zero ();
        }

        if (key == 0xFFA857 /*DOWN_KEY*/) {
          // beep ();
          yer--;
          if (yer < 20) {
            yer = 99;
          }
          bL_to_zero ();
        }

        if (key == 0xFF02FD /*OK_KEY*/) {
          lamps [5] = yer % 10;
          lamps [4] = yer / 10;
          break;
        }
      }
      datetime.Day = _day;
      datetime.Month = mon;
      datetime.Year = CalendarYrToTm(20 * 100 + yer);
      RTC.write(datetime);

      goto exit_cycle;
    }
    // Конец цикла настройки даты

    // Если же нажали кнопку настройки находясь в режиме показа времени, то будем настраивать время
    s = 0;
    setup_mode = 1;
    delay (100);
    while (1) {
      lamps [5] = 0;
      lamps [4] = 0;

      lamps [3] = m % 10;
      lamps [2] = m / 10;

      key = return_code ();
      if (millis() - bL1 >= 100) {  // ищем разницу (500 мс)
        bL1 = millis();
        lamps [1] = h % 10;
        lamps [0] = h / 10;
      }
      if (millis() - bL2 >= 200) {
        bL2 = millis();
        lamps [1] = 10;
        lamps [0] = 10;
      }

      if (key_pressed (A2, 0, h, true) || (key == 0xFF629D /*UP_KEY*/)) {
        h++;
        if (h > 23) {
          h = 0;
        }
        bL_to_zero ();
      }

      if (key_pressed (A0, 0, h, true) || (key == 0xFFA857 /*DOWN_KEY*/)) {
        //beep ();
        h--;
        if (h > 23) {
          h = 23;
        }
        bL_to_zero ();
      }

      if (key_pressed (A1, 0, 0, false) || (key == 0xFF02FD /*OK_KEY*/)) {
        break;
      }
    }


    bL_to_zero ();
    while (1) {
      lamps [5] = 0;
      lamps [4] = 0;

      lamps [1] = h % 10;
      lamps [0] = h / 10;

      key = return_code ();
      if (millis() - bL1 >= 100) {  // ищем разницу (500 мс)
        bL1 = millis();
        lamps [3] = m % 10;
        lamps [2] = m / 10;
      }
      if (millis() - bL2 >= 200) {
        bL2 = millis();
        lamps [3] = 10;
        lamps [2] = 10;
      }

      if (key_pressed (A2, 2, m, true) || (key == 0xFF629D /*UP_KEY*/)) {
        m++;
        if (m > 59) {
          m = 0;
        }
        bL_to_zero ();
      }

      if (key_pressed (A0, 2, m, true) || (key == 0xFFA857 /*DOWN_KEY*/)) {
        m--;
        if (m > 59) {
          m = 59;
        }
        bL_to_zero ();
      }

      if (key_pressed (A1, 0, 0, false) || (key == 0xFF02FD /*OK_KEY*/)) {
        bL_to_zero ();
        setup_mode = 0;

        datetime.Hour = h;
        datetime.Minute = m;
        datetime.Second = 0;
        RTC.write(datetime); // записать данные в DS3231
        goto exit_cycle;
      }
    }
  }
exit_cycle:
  //************************************ КОНЕЦ ************************************


  if (key == 0xFF42BD) { //KEY_*
    vol_down ();
  }

  if (key == 0xFF52AD) { //KEY_#
    vol_up ();
  }

  if (key == 0xFF4AB5) { //KEY_0
    if (!radio_on) {
      switch_on_radio ();
      set_volume (0);
      vol = 0;
      radio_on = 1;
    } else {
      shutdown_radio ();
      radio_on = 0;
    }
  }

  if (key == 0xFF6897) { //KEY_1
    freq = f1;
    chg = setup_freq ();
    f1 = freq;
    if (chg) {
      eeprom_write_word (&f1_addr, f1);
    }
    chg = 0;
  }

  if (key == 0xFF9867) { //KEY_2
    freq = f2;
    chg = setup_freq ();
    f2 = freq;
    if (chg) {
      eeprom_write_word (&f2_addr, f2);
    }
    chg = 0;
  }

  if (key == 0xFFB04F) { //KEY_3
    freq = f3;
    chg = setup_freq ();
    f3 = freq;
    if (chg) {
      eeprom_write_word (&f3_addr, f3);
    }
    chg = 0;
  }

  if (key == 0xFF30CF) { //KEY_4
    freq = f4;
    chg = setup_freq ();
    f4 = freq;
    if (chg) {
      eeprom_write_word (&f4_addr, f4);
    }
    chg = 0;
  }

  if (key == 0xFF18E7) { //KEY_5
    freq = f5;
    chg = setup_freq ();
    f5 = freq;
    if (chg) {
      eeprom_write_word (&f5_addr, f5);
    }
    chg = 0;
  }

  if (key == 0xFF7A85) { //KEY_6
    freq = f6;
    chg = setup_freq ();
    f6 = freq;
    if (chg) {
      eeprom_write_word (&f6_addr, f6);
    }
    chg = 0;
  }

  //************************************ ЦИКЛ НАСТРОЙКИ РЕЖИМОВ (ПРОКРУТКА,  ТИК, РЕВЕРС, БУДИЛЬНИК) ************************************
  if (key == 0xFF629D /*UP_KEY*/) {
    while (1) {
      lamps [5] = sec_scroll;
      lamps [4] = 10;
      lamps [3] = 1;
      lamps [2] = 10;
      lamps [1] = 10;
      lamps [0] = 10;
      key = return_code ();
      if (key == 0xFF02FD /*OK_KEY*/) {
        break;
      }

      if (key == 0xFF629D /*UP_KEY*/) {
        if (sec_scroll == 1) {
          sec_scroll = 0;
        } else {
          sec_scroll = 1;
        }
      }
    }

    while (1) {
      lamps [5] = all_30_scroll;
      lamps [4] = 10;
      lamps [3] = 2;
      lamps [2] = 10;
      lamps [1] = 10;
      lamps [0] = 10;
      key = return_code ();
      if (key == 0xFF02FD /*OK_KEY*/) {
        break;
      }

      if (key == 0xFF629D /*UP_KEY*/) {
        if (all_30_scroll == 1) {
          all_30_scroll = 0;
        } else {
          all_30_scroll = 1;
        }
      }
    }

    while (1) {
      lamps [5] = tik_tak;
      lamps [4] = 10;
      lamps [3] = 3;
      lamps [2] = 10;
      lamps [1] = 10;
      lamps [0] = 10;
      key = return_code ();
      if (key == 0xFF02FD /*OK_KEY*/) {
        break;
      }

      if (key == 0xFF629D /*UP_KEY*/) {
        if (tik_tak == 1) {
          tik_tak = 0;
        } else {
          tik_tak = 1;
        }
      }
    }

    while (1) {
      lamps [5] = reverse;
      lamps [4] = 10;
      lamps [3] = 4;
      lamps [2] = 10;
      lamps [1] = 10;
      lamps [0] = 10;
      key = return_code ();
      if (key == 0xFF02FD /*OK_KEY*/) {
        break;
      }

      if (key == 0xFF629D /*UP_KEY*/) {
        if (reverse == 1) {
          reverse = 0;
        } else {
          reverse = 1;
        }
      }
    }

    while (1) {
      lamps [5] = defect;
      lamps [4] = 10;
      lamps [3] = 5;
      lamps [2] = 10;
      lamps [1] = 10;
      lamps [0] = 10;
      key = return_code ();
      if (key == 0xFF02FD /*OK_KEY*/) {
        break;
      }

      if (key == 0xFF629D /*UP_KEY*/) {
        if (defect == 1) {
          defect = 0;
        } else {
          defect = 1;
        }
      }
    }

    while (1) {
      lamps [5] = alarm_is_active;
      lamps [4] = 10;
      lamps [3] = 6;
      lamps [2] = 10;
      lamps [1] = 10;
      lamps [0] = 10;
      key = return_code ();
      if (key == 0xFF02FD /*OK_KEY*/) {
        break;
      }

      if (key == 0xFF629D /*UP_KEY*/) {
        if (alarm_is_active == 1) {
          alarm_is_active = 0;
        } else {
          alarm_is_active = 1;
        }
      }
    }

    if (alarm_is_active == 1) {
      while (1) {
        lamps [0] = alarm_clock_hour / 10;
        lamps [1] = alarm_clock_hour % 10;
        lamps [2] = alarm_clock_minute / 10;
        lamps [3] = alarm_clock_minute % 10;
        lamps [4] = 10;
        lamps [5] = 10;

        key = return_code ();

        if (key == 0xFF629D /*UP_KEY*/) {
          alarm_clock_hour++;
          if (alarm_clock_hour > 23) alarm_clock_hour = 0;
        }

        if (key == 0xFFA857 /*DOWN_KEY*/) {
          alarm_clock_hour--;
          if (alarm_clock_hour > 23) alarm_clock_hour = 23;
        }


        if (key == 0xFFC23D /*RIGHT_KEY*/) {
          alarm_clock_minute++;
          if (alarm_clock_minute > 59) alarm_clock_minute = 0;
        }

        if (key == 0xFF22DD /*LEFT_KEY*/) {
          alarm_clock_minute--;
          if (alarm_clock_minute > 59) alarm_clock_minute = 59;
        }

        if (key == 0xFF02FD /*OK_KEY*/) {
          eeprom_write_byte (&alarm_clock_hour_addr, alarm_clock_hour);
          eeprom_write_byte (&alarm_clock_minute_addr, alarm_clock_minute);
          break;
        }
      }
    }

    eeprom_write_byte(&sec_scroll_addr, sec_scroll);
    eeprom_write_byte(&all_30_scroll_addr, all_30_scroll);
    eeprom_write_byte(&tik_tak_addr, tik_tak);
    eeprom_write_byte(&reverse_addr, reverse);
    eeprom_write_byte(&alarm_is_active_addr, alarm_is_active);
    eeprom_write_byte(&defect_addr, defect);
  }
  //************************************ КОНЕЦ ************************************

  if (key_pressed (A2, 0, 0, false) || (key == 0xFFC23D /*RIGHT_KEY*/)) { //
    num_of_effect = 0;
    color += 10;
    FastLED.setBrightness (30);
    for (unsigned char i = 0; i <= NUM_LEDS; i++) {
      leds_0 [i] = CHSV(color, 255, 255);;
    }
    FastLED.show();
  }
  if (key_pressed (A0, 0, 0, false) || (key == 0xFF22DD /*LEFT_KEY*/)) {
    num_of_effect++;
    if (num_of_effect > 4) {
      num_of_effect = 0;
    }

    if (num_of_effect == 0) {
      off_backlight ();
    }
  }

  if (key == 0xFFA857 /*DOWN_KEY*/) {
    show_date = 1;

    old_effects = sec_scroll;
    sec_scroll = 0;

    lamps [6] = yer % 10;
    lamps [7] = yer / 10;
    lamps [8] = mon / 10;
    lamps [9] = mon % 10;
    lamps [10] = _day % 10;
    lamps [11] = _day / 10;
    for (int m = 0; m <= 5; m++) {
      int tt = lamps [0];
      for (int n = 1; n <= 11; n++) {
        lamps [n - 1] = lamps [n];
      }
      lamps [11] = tt;
      delay (100);
    }

    bL2 = millis (); // Это надо, иначе дата показываться не будет. Нужно для задержки.
  }

  /*
    if ((h == 2) && (m == 30) && (s == 0)) {
      night_cycle = 1;
      old_n_effects = 0;
      old_n_effects = sec_scroll;
      old_n_effects *= 10;
      old_n_effects += all_30_scroll;
      off_backlight ();
      all_30_scroll = 0;
      sec_scroll = 0;
    }
    if ((h == 4) && (m == 30) && (s == 0)) {
      night_cycle = 0;
      all_30_scroll = old_n_effects % 10;
      sec_scroll = old_n_effects / 10;
    }
  */
  if (key == 0xFF38C7) {

    old_n_effects = 0;
    old_n_effects = sec_scroll;
    old_n_effects *= 10;
    old_n_effects += all_30_scroll;

    all_30_scroll = 0;
    sec_scroll = 0;

    tempera = GetTemp ();
    while (1) {
      key = return_code ();
      lamps [5] = 10;
      lamps [4] = 10;
      lamps [2] = (tempera / 10) % 10;
      lamps [3] = tempera % 10;
      lamps [1] = 10;
      lamps [0] = 10;
      if (key == 0xFF38C7) break;
    }

    all_30_scroll = old_n_effects % 10;
    sec_scroll = old_n_effects / 10;
  }

  if (num_of_effect == 1) {
    FastLED.setBrightness (30);
    static uint32_t prevTime;
    // двигаем пламя
    if (millis() - prevTime > 20) {
      prevTime = millis();
      //int thisPos = 0, lastPos = 0;
      for (int i = 0; i < NUM_LEDS; i++) {
        leds_0[i] = getFireColor((inoise8(i * FIRE_STEP, counter)));
      }
      counter += 20;
      FastLED.show();
      if (return_code () == 0xFF22DD) {
        num_of_effect = 2;
      };
    }
  }

  if (num_of_effect == 2) {
    rainbow_loop ();
  }

  if (num_of_effect == 3) {
    color_bounceFADE();
  }

  if (num_of_effect == 4) {
    color_loop_vardelay ();
  }

  if ((m % 5 == 0) && (s == 0)) {
    update_time_date ();
  }
  if ((millis() - myTimer1 >= 1000) && (!setup_mode)) {  // ищем разницу (500 мс)
    myTimer1 = millis();               // сброс таймера
    s++;

    if (all_30_scroll == 1) {
      if (s % 30 == 0) {
        for (unsigned char t = 0; t <= 9; t++) {
          lamps [0] = t;
          lamps [1] = t;
          lamps [2] = t;
          lamps [3] = t;
          lamps [4] = t;
          lamps [5] = t;
          delay (20);
        }
        show_time (h, m, s);
      }
    }

    if ((sec_scroll == 1) && (!show_date)) {
      scrolling (5);
      if (s % 10 == 0) scrolling (4);
    }
    if (tik_tak == 1) {
      beep ();
    }
    if (s > 59) {
      s = 0;
      m++;
      if ((sec_scroll == 1) && (!show_date)) {
        scrolling (3);
        if (m % 10 == 0) scrolling (2);
      }
      if (m > 59) {
        m = 0;
        s = 0;
        h++;
        if (sec_scroll == 1) {
          scrolling (1);
          if (h % 10 == 0) scrolling (0);
        }
        if (h > 23) {
          h = 0;
          m = 0;
          s = 0;
        }
      }
    }
  }

  if (alarm_is_active == 1) {
    if (alarm_is_already_on == 1) {
      if (millis () % 10000 == 0) {
        vol_up ();
        delay (1);
      }
    }
    if ((h == alarm_clock_hour) && (m == alarm_clock_minute) && (s == 0)) {
      freq = f1;
      setFreq (freq);
      set_volume (0);
      vol = 0;
      switch_on_radio ();
      s += 2;
      alarm_is_already_on = 1;
    }
  }

  // ============= блок вывода минут, секунд и часов в массив lamps, который затем прервываниями выводится на лампы.
  if  ((!setup_mode) && (!night_cycle) && (!show_date)) {
    if (reverse == 0) {
      show_time (h, m, s);
    } else {
      show_time (23 - h, 59 - m, 59 - s);
    }
  }

  if (show_date == 1) {
    // ************************ ЕСЛИ ВКЛЮЧЕНА ДАТА И НАЖАТА ОК, ВКЛЮЧАЕМ ЦИКЛ НАСТРОЙКИ ДАТЫ ************************
    bL1 = 0;
    // ********************** КОНЕЦ ЦИКЛА НАСТРОЙКИ ДАТЫ **********************

    if (millis() - bL2 >= 5000) {
      show_date = 0;

      if (!reverse) {
        lamps [6] = h / 10;
        lamps [7] = h % 10;
        lamps [8] = m / 10;
        lamps [9] = m % 10;
        lamps [10] = s / 10;
        lamps [11] = s % 10;
      }
      else {
        lamps [6] = (23 - h) / 10;
        lamps [7] = (23 - h) % 10;
        lamps [8] = (59 - m) / 10;
        lamps [9] = (59 - m) % 10;
        lamps [10] = (59 - s) / 10;
        lamps [11] = (59 - s) % 10;
      }
      for (int m = 0; m <= 5; m++) {
        int tt = lamps [0];
        for (int n = 1; n <= 11; n++) {
          lamps [n - 1] = lamps [n];
        }
        lamps [11] = tt;
        delay (100);
      }
      update_time_date ();
    }
    sec_scroll = old_effects;
    show_time (_day, mon, yer);
  }

  if ((h == 2) && (m == 30) && (s == 0)) {
    night_cycle = 1;
    old_n_effects = 0;
    old_n_effects = sec_scroll;
    old_n_effects *= 10;
    old_n_effects += all_30_scroll;
    off_backlight ();
    all_30_scroll = 0;
    sec_scroll = 0;
  }
  if ((h == 4) && (m == 30) && (s == 0)) {
    night_cycle = 0;
    all_30_scroll = old_n_effects % 10;
    sec_scroll = old_n_effects / 10;
  }

  if (night_cycle) {
    if (!temporary_show) {
      lamps [5] = (millis () / 100) % 10;
      lamps [4] = (millis () / 100) % 10;
      lamps [3] = (millis () / 100) % 10;
      lamps [2] = (millis () / 100) % 10;
      lamps [1] = (millis () / 100) % 10;
      lamps [0] = (millis () / 100) % 10;
    }
    if (return_code () > 0) {
      show_time (h, m, s);
      temporary_show = 1;
    }
    if (temporary_show) {
      tmp++;
      show_time (h, m, s);
      if (tmp > 50000) {
        tmp = 0;
        temporary_show = 0;
      }
    }
  }
}


uint8_t GetTemp(void)
{
  unsigned int wADC;
  double t;
  uint8_t tempera;

  // The internal temperature has to be used
  // with the internal reference of 1.1V.
  // Channel 8 can not be selected with
  // the analogRead function yet.

  // Set the internal reference and mux.
  ADMUX = (_BV(REFS1) | _BV(REFS0) | _BV(MUX3));
  ADCSRA |= _BV(ADEN);  // enable the ADC

  delay(20);            // wait for voltages to become stable.

  ADCSRA |= _BV(ADSC);  // Start the ADC

  // Detect end-of-conversion
  while (bit_is_set(ADCSRA, ADSC));

  // Reading register "ADCW" takes care of how to read ADCL and ADCH.
  wADC = ADCW;

  // The offset of 324.31 could be wrong. It is just an indication.
  t = (wADC - 292.31 ) / 1.22;

  // The returned temperature is in degrees Celsius.
  tempera = round (t);
  return (tempera);
}

// возвращает цвет огня для одного пикселя
CRGB getFireColor(int val) {
  // чем больше val, тем сильнее сдвигается цвет, падает насыщеность и растёт яркость
  return CHSV(
           HUE_START + map(val, 0, 255, 0, HUE_GAP),                    // H
           constrain(map(val, 0, 255, MAX_SAT, MIN_SAT), 0, 255),       // S
           constrain(map(val, 0, 255, MIN_BRIGHT, MAX_BRIGHT), 0, 255)  // V
         );
}

void rainbow_loop() {                        //-m3-LOOP HSV RAINBOW
  idex++;
  ihue = ihue + thisstep;
  if (idex >= NUM_LEDS) {
    idex = 0;
  }
  if (ihue > 255) {
    ihue = 0;
  }
  leds_0[idex] = CHSV(ihue, thissat, 255);
  FastLED.show();
  delay(50);
}

void show_time (unsigned char a, unsigned char b, unsigned char c) {
  if (defect) {
    defect_lamp = random (50);
    if ((random (300) == 298) && (defect_lamp >= 0) && (defect_lamp <= 5)) {
      lamps [defect_lamp] = 10;
      delay (50);
    }
  }
  lamps [5] = c % 10;
  lamps [4] = c / 10;
  lamps [3] = b % 10;
  lamps [2] = b / 10;
  lamps [1] = a % 10;
  lamps [0] = a / 10;
}

int antipodal_index(int i) {
  int iN = i + TOP_INDEX;
  if (i >= TOP_INDEX) {
    iN = ( i + TOP_INDEX ) % NUM_LEDS;
  }
  return iN;
}


//---FIND ADJACENT INDEX CLOCKWISE
int adjacent_cw(int i) {
  int r;
  if (i < NUM_LEDS - 1) {
    r = i + 1;
  }
  else {
    r = 0;
  }
  return r;
}

//---FIND ADJACENT INDEX COUNTER-CLOCKWISE
int adjacent_ccw(int i) {
  int r;
  if (i > 0) {
    r = i - 1;
  }
  else {
    r = NUM_LEDS - 1;
  }
  return r;
}

void color_loop_vardelay() {                    //-m17-COLOR LOOP (SINGLE LED) w/ VARIABLE DELAY
  idex++;
  if (idex > NUM_LEDS) {
    idex = 0;
  }

  for (int i = 0; i < NUM_LEDS; i++ ) {
    if (i == idex) {
      leds_0[i] = CHSV(random (0, 255), thissat, 255);
    }
    else {
      leds_0[i].r = 0; leds_0[i].g = 0; leds_0[i].b = 0;
    }
  }
  FastLED.show();
  delay (100);
}

void color_bounceFADE() {                    //-m6-BOUNCE COLOR (SIMPLE MULTI-LED FADE)
  if (bouncedirection == 0) {
    idex = idex + 1;
    if (idex == NUM_LEDS) {
      bouncedirection = 1;
      idex = idex - 1;
    }
  }
  if (bouncedirection == 1) {
    idex = idex - 1;
    if (idex == 0) {
      bouncedirection = 0;
    }
  }
  int iL1 = adjacent_cw(idex);
  int iL2 = adjacent_cw(iL1);
  int iL3 = adjacent_cw(iL2);
  int iR1 = adjacent_ccw(idex);
  int iR2 = adjacent_ccw(iR1);
  int iR3 = adjacent_ccw(iR2);
  for (int i = 0; i < NUM_LEDS; i++ ) {
    if (i == idex) {
      leds_0[i] = CHSV(random (0, 255), thissat, 255);
    }
    else if (i == iL1) {
      leds_0[i] = CHSV(random (0, 255), thissat, 150);
    }
    else if (i == iL2) {
      leds_0[i] = CHSV(random (0, 255), thissat, 80);
    }
    else if (i == iL3) {
      leds_0[i] = CHSV(random (0, 255), thissat, 20);
    }
    else if (i == iR1) {
      leds_0[i] = CHSV(random (0, 255), thissat, 150);
    }
    else if (i == iR2) {
      leds_0[i] = CHSV(random (0, 255), thissat, 80);
    }
    else if (i == iR3) {
      leds_0[i] = CHSV(random (0, 255), thissat, 20);
    }
    else {
      leds_0[i] = CHSV(0, 0, 0);
    }
  }
  FastLED.show();
  delay (100);
}

long return_code (void) {
  // читаем код кнопки с пду
  long ret;

  if ( irrecv.decode( &results )) {
    if (alarm_is_already_on == 1) {
      alarm_is_already_on = 0;
      vol = 0;
      set_volume (0);
      mute_radio ();
      delay (1000);
      update_time_date ();
      irrecv.resume();
      return 0;
    }
    ret = results.value;
    irrecv.resume();
    beep ();
    return ret;
  } else {
    return 0;
  }
}

void off_backlight (void) {
  FastLED.setBrightness (0);
  for (unsigned char i = 0; i <= NUM_LEDS; i++) {
    leds_0 [i] = CHSV(0, 0, 0);;
  }
  FastLED.show();
}

void show_lamp (unsigned char n) {
  if (n <= 9) {
    digitalWrite (5, dig[n] & 0b1000);
    digitalWrite (6, dig[n] & 0b100);
    digitalWrite (7, dig[n] & 0b10);
    digitalWrite (8, dig[n] & 0b1);
  } else {
    digitalWrite (5, 1);
    digitalWrite (6, 1);
    digitalWrite (7, 1);
    digitalWrite (8, 1);
  }
}

void beep () {
  for (char t = 1; t < 50; t++) {
    digitalWrite (11, 1);
    delayMicroseconds (10);
    digitalWrite (11, 0);
    delayMicroseconds (10);
  }
}

void scrolling (unsigned char n) {
  if (sec_scroll == 1) {
    for (int t = 0; t <= 9; t++) {
      lamps [n] = t;
      delay (20);
    }
  }
}

void update_time_date () {
  RTC.read(datetime);
  s = datetime.Second;
  m = datetime.Minute;
  h = datetime.Hour;

  _day = datetime.Day % 100;
  mon = datetime.Month % 100;
  yer = tmYearToCalendar(datetime.Year) % 100;
}

void bL_to_zero () {
  bL1 = 0;
  bL2 = 0;
}

bool setup_freq () {
  lamps [0] = 10;
  lamps [1] = 10;
  lamps [2] = (freq / 1000) % 10;
  lamps [3] = (freq / 100) % 10;
  lamps [4] = (freq / 10) % 10;
  lamps [5] = freq % 10;

  bL_to_zero ();

  bool is_changed = 0;
  unsigned char blk = 2;
  unsigned char tmp = 0;

  tmp = lamps [blk];

  while (1) {
    key = return_code ();

    if (millis() - bL1 >= 100) {  // ищем разницу (500 мс)
      bL1 = millis();
      lamps [blk] = tmp;
    }
    if (millis() - bL2 >= 200) {
      bL2 = millis();
      lamps [blk] = 10;
    }

    if (key == 0xFF02FD) { //KEY_OK
      update_time_date ();

      lamps [blk] = tmp;

      freq = lamps [2] * 10;
      freq += lamps [3];
      freq *= 10;
      freq += lamps [4];
      freq *= 10;
      freq += lamps [5];

      if (freq > 1080) {
        freq = 1080;
      }
      if (freq < 875) {
        freq = 875;
      }

      setFreq (freq);

      switch_on_radio ();
      radio_on = 1;

      return is_changed;
    }

    if (key == 0xFF22DD) { //KEY_LEFT
      lamps [blk] = tmp;
      blk--;
      if (blk < 2) blk = 5;
      tmp = lamps [blk];
      bL_to_zero ();
    }

    if (key == 0xFFC23D) { //KEY_RIGHT
      lamps [blk] = tmp;
      blk++;
      if (blk > 5) blk = 2;
      tmp = lamps [blk];
      bL_to_zero ();
    }

    if (key == 0xFF629D) { //KEY_UP
      tmp++;
      if (blk == 2) {
        if (tmp > 1) tmp = 0;
      } else {
        if (tmp > 9) tmp = 0;
      }
      bL_to_zero ();
      is_changed = 1;
    }

    if (key == 0xFFA857) { //KEY_DOWN
      tmp--;
      if (blk == 2) {
        if (tmp > 9) tmp = 1;
      } else {
        if (tmp > 9) tmp = 9;
      }
      bL_to_zero ();
      is_changed = 1;
    }
  }
}
